package com.nokia.common.gpload;

import com.nokia.common.gpload.entity.GploadResult;
import com.xxl.job.core.context.XxlJobHelper;
import lombok.extern.slf4j.Slf4j;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

@Slf4j
public class GploadUtil {

    private static Process process = null;
    private static BufferedReader reader = null;

    public static GploadResult gpload(String gploadCommand) {
        GploadResult result = new GploadResult();
        try {
            Process process = Runtime.getRuntime().exec(gploadCommand);
            reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
            List<String> lines = new ArrayList<>();
            String line;
            while ((line = reader.readLine()) != null) {
                lines.add(line);
            }
            for (int i = 0; i < lines.size(); i++) {
                if (lines.get(i).contains("|INFO|rows Inserted")) {
                    // 找到第一行 对应的是插入数据的数量
                    // s4可以判断gpload是否成功
                    String s4 = lines.get(i + 3);
                    if (s4.endsWith("succeeded")) {
                        // 插入数量
                        String s1 = lines.get(i);
                        s1 = s1.substring(s1.indexOf('=') + 2);
                        // 更新数量
                        String s2 = lines.get(i + 1);
                        s2 = s2.substring(s2.indexOf('=') + 2);
                        // 错误数量
                        String s3 = lines.get(i + 2);
                        s3 = s3.substring(s3.indexOf('=') + 2);
                        result.setInsertedCount(Integer.parseInt(s1));
                        result.setUpdatedCount(Integer.parseInt(s2));
                        result.setErrorCount(Integer.parseInt(s3));
                    } else {
                        StringBuilder sb = new StringBuilder();
                        for (int j = i - 1; j > 0; j--) {
                            sb.insert(0, lines.get(i));
                            if (lines.get(i).contains("|ERROR|ERROR:")) {
                                sb.insert(0, lines.get(i));
                                break;
                            }
                        }
                        result.setMessage(sb.toString());
                    }
                }
            }
            int exitCode = process.waitFor();
            result.setTaskStatus(exitCode == 0);
            if (Thread.interrupted()) {
                log.error("gpload线程阻塞");
                XxlJobHelper.log("gpload线程阻塞");
                throw new InterruptedException();
            }
        } catch (IOException | InterruptedException e) {
            log.debug("gpload发生异常: {}", e.getMessage(), e);
            XxlJobHelper.log("gpload发生异常: {}", e.getMessage(), e);
            result.setTaskStatus(false);
            result.setMessage(e.getMessage());
            if (e instanceof InterruptedException) {
                Thread.currentThread().interrupt();
            }
        } finally {
            log.debug("gpload的结果为: {}", result);
            XxlJobHelper.log("gpload的结果为: {}", result);
            destroy();
        }
        return result;
    }

    private static void destroy() {
        if (reader != null) {
            try {
                reader.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            reader = null;
        }
        if (process != null) {
            process.destroy();
            process = null;
        }
    }
}
