package com.nokia.pm_interface_5g;

import com.nokia.pm_interface_5g.task.FiveGPmTask;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVPrinter;
import org.apache.commons.csv.CSVRecord;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;

@Slf4j
@SpringBootTest
class PmInterface5gApplicationTests {
    @Autowired
    private FiveGPmTask fiveGPmTask;

    @Test
    void testRerrange() {
        String readPath = "a/pm_5g_hour_2022072912.csv";
        String resultPath = "a/";
        try {
            Files.createDirectories(Paths.get(resultPath));
        } catch (IOException e) {
            e.printStackTrace();
            return;
        }
        String writePath = resultPath + "result.csv";
        try (CSVParser parser = CSVFormat.DEFAULT.builder().build()
                .parse(new InputStreamReader(Files.newInputStream(Paths.get(readPath)), StandardCharsets.UTF_8));
             OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(writePath)),
                     StandardCharsets.UTF_8);
             CSVPrinter printer = new CSVPrinter(osw, CSVFormat.DEFAULT);) {
            for (CSVRecord t : parser) {
                String nciStr = t.get(2);
                String enbid = "";
                String cellId = "";
                if (!StringUtils.hasText(nciStr)) {
                    continue;
                }
                if (nciStr.contains(".")) {
                    String[] a = nciStr.split("\\.");
                    enbid = a[1];
                    cellId = a[2];
                } else if (nciStr.contains("-")) {
                    String[] a = nciStr.split("-");
                    enbid = a[1];
                    cellId = a[2];
                }
                if (!StringUtils.hasText(enbid) || !StringUtils.hasText(cellId)) {
                    continue;
                }
                String enbCell = enbid + "_" + cellId;
                int nci = Integer.parseInt(enbid) * 4096 + Integer.parseInt(cellId);
                printer.printRecord(
                        t.get(1),
                        t.get(3),
                        nci,
                        enbid,
                        cellId,
                        enbCell,
                        t.get(6),
                        format(t.get(10)),
                        format(t.get(18)),
                        format(t.get(14)),
                        format(t.get(23)),
                        format(t.get(79)),
                        format(t.get(59)),
                        format(t.get(60)),
                        format(t.get(53)),
                        format(t.get(56)),
                        format(t.get(80)),
                        format(t.get(11)),
                        format(t.get(81)),
                        format(t.get(82)),
                        format(t.get(83)),
                        format(t.get(84)),
                        format(t.get(85)),
                        format(t.get(86)),
                        format(t.get(87)),
                        format(t.get(88)),
                        format(t.get(89)),
                        format(t.get(90)),
                        format(t.get(91)),
                        format(t.get(92)),
                        format(t.get(93)),
                        format(t.get(94)),
                        format(t.get(95)),
                        format(t.get(96)),
                        format(t.get(97)),
                        format(t.get(98)),
                        format(t.get(99)),
                        format(t.get(100)),
                        format(t.get(101)),
                        format(t.get(102)),
                        format(t.get(103)),
                        format(t.get(104)),
                        format(t.get(62)),
                        format(t.get(63)),
                        format(t.get(105)),
                        format(t.get(106)),
                        format(t.get(57)),
                        format(t.get(58)),
                        format(t.get(107)),
                        format(t.get(108)),
                        format(t.get(109)),
                        format(t.get(110)),
                        t.get(111),
                        t.get(112),
                        t.get(113),
                        t.get(8),
                        t.get(7),
                        format(t.get(20)),
                        format(t.get(19)),
                        format(t.get(12)),
                        format(t.get(13)),
                        format(t.get(114)),
                        format(t.get(115)),
                        format(t.get(116)),
                        t.get(117),
                        t.get(118),
                        t.get(119),
                        t.get(120),
                        t.get(121),
                        t.get(122),
                        t.get(123),
                        t.get(124),
                        format(t.get(125)),
                        format(t.get(126)),
                        format(t.get(59)),
                        format(t.get(60)),
                        format(t.get(127)),
                        format(t.get(128)),
                        format(t.get(129)),
                        format(t.get(130)),
                        format(t.get(131)),
                        format(t.get(132)),
                        format(t.get(133)),
                        t.get(134),
                        t.get(135),
                        t.get(136),
                        t.get(137),
                        t.get(138),
                        t.get(139),
                        t.get(140),
                        t.get(141),
                        format(t.get(142)),
                        format(t.get(143)),
                        t.get(144),
                        t.get(145),
                        t.get(147),
                        t.get(148),
                        t.get(149),
                        t.get(150),
                        t.get(151),
                        t.get(152),
                        t.get(153),
                        t.get(154),
                        t.get(155),
                        t.get(156),
                        t.get(157),
                        t.get(158),
                        t.get(159),
                        t.get(160),
                        format(t.get(161)),
                        format(t.get(162)),
                        format(t.get(163)),
                        format(t.get(164)),
                        format(t.get(165)),
                        format(t.get(57)),
                        format(t.get(58)),
                        format(t.get(41)),
                        format(t.get(166)),
                        t.get(167),
                        t.get(168),
                        format(t.get(169)),
                        format(t.get(65)),
                        format(t.get(64)),
                        format(t.get(61))
                );
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    BigDecimal format(String s) {
        if (!StringUtils.hasText(s)) {
            return null;
        }

        return new BigDecimal(s).setScale(4, RoundingMode.UP);
    }
}
